# (c) Copyright 2011 Cloudera, Inc.

import re
import datetime
import time
from event import LogEvent

MONTHS = {"Jan":1,"Feb":2,"Mar":3,"Apr":4, "May":5, "Jun":6,
"Jul":7,"Aug":8,"Sep":9,"Oct":10,"Nov":11,"Dec":12}

PYTHON_LOG_LEVELS = ["DEBUG", "DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]

# Contains both GLog and Log4J log levels
GLOG_LOG_LEVELS = {"I":"INFO", "W":"WARN", "E":"ERROR", "F":"FATAL",
"DEBUG":"DEBUG", "TRACE":"TRACE", "INFO": "INFO", "WARN":"WARN",
"ERROR":"ERROR", "FATAL":"FATAL"}

class Log4JEventParser(object):

    def __init__(self):
        date_format = "(?P<year>\d{4})-(?P<month>\d{2})-(?P<day>\d{2})"
        time_format = "(?P<hour>\d{2}):(?P<minute>\d{2}):(?P<second>\d{2}),(?P<ms>\d{3})"
        log_level_format = "\w+"
        source_format = "\S+"
        message_format = ".*\\n{0,1}"

        log4jregex_string = "(?P<date>" + date_format + \
            ")\s+(?P<time>" + time_format + \
            ")\s+(?P<loglevel>" + log_level_format + \
            ")\s+(?P<source>" + source_format + \
            "):\s*(?P<message>" + message_format + ")"

        self.__log4jregex = re.compile(log4jregex_string)

    def parse_event(self, line):
        m = self.__log4jregex.match(line.get_content())
        if m is not None:
            return self.__construct_event(
                line.get_file_path(), line.get_offset(),
                line.get_offset() + len(line.get_content()), m)
        else:
            return None

    def __construct_event(self, file_path, offset, end_offset, m):
        dt = datetime.datetime(
            int(m.group("year")), int(m.group("month")), int(m.group("day")),
            int(m.group("hour")), int(m.group("minute")), int(m.group("second")),
            int(m.group("ms")) * 1000)
        return LogEvent(
            file_path, offset, end_offset, dt, m.group("loglevel"),
            m.group("source"), m.group("message"))

class HueEventParser(object):

    def __init__(self):
        date_format = "\[(?P<day>\d{2})/(?P<month>\w{3})/(?P<year>\d{4})"
        time_format = "(?P<hour>\d{2}):(?P<minute>\d{2}):(?P<second>\d{2})\s+[+-]\d{4}\]"
        log_level_format = "\w+"
        source_format = "[\w+\.]+"
        message_format = ".+\\n{0,1}"

        hue_regex_string = "(?P<date>" + date_format + \
            ")\s+(?P<time>" + time_format + \
            ")\s+(?P<source>" + source_format + \
            ")\s+(?P<loglevel>" + log_level_format + \
            ")\s+(?P<message>" + message_format + ")"

        self.__hue_regex = re.compile(hue_regex_string)

    def parse_event(self, line):
        m = self.__hue_regex.match(line.get_content())
        if m is not None:
            return self.__construct_event(
                line.get_file_path(), line.get_offset(),
                line.get_offset() + len(line.get_content()), m)
        else:
            return None

    def __construct_event(self, file_path, offset, end_offset, m):
        dt = datetime.datetime(
            int(m.group("year")), MONTHS[m.group("month")], int(m.group("day")),
            int(m.group("hour")), int(m.group("minute")), int(m.group("second")))

        # Hue logs are in UTC, change to local time
        utc_delta = datetime.timedelta(seconds=time.altzone)
        dt -= utc_delta

        return LogEvent(
            file_path, offset, end_offset, dt, m.group("loglevel"),
            m.group("source"), m.group("message"))


class ScmAgentEventParser(object):
    
    def __init__(self):
        date_format = "\[(?P<day>\d{2})/(?P<month>\w{3})/(?P<year>\d{4})"
        time_format = "(?P<hour>\d{2}):(?P<minute>\d{2}):(?P<second>\d{2})\s+[+-]\d{4}\]"
 
        # In the agent logs the thread name can have spaces and hyphens. We want to capture the entire
        # string and stop when we find a valid log level. 
        source_format = ".+[\w\,\-]+"
        log_level_format = "|".join(PYTHON_LOG_LEVELS)
        message_format = ".+\\n{0,1}"

        agent_regex_string = "(?P<date>" + date_format + \
            ")\s+(?P<time>" + time_format + \
            ")\s+(?P<source>" + source_format + \
            ")\s+(?P<loglevel>" + log_level_format + \
            ")\s+(?P<message>" + message_format + ")"

        self.__agent_regex = re.compile(agent_regex_string)

    def parse_event(self, line):
        m = self.__agent_regex.match(line.get_content())
        if m is not None:
            return self.__construct_event(
                line.get_file_path(), line.get_offset(),
                line.get_offset() + len(line.get_content()), m)
        else:
            return None

    def __construct_event(self, file_path, offset, end_offset, m):
        dt = datetime.datetime(
            int(m.group("year")), MONTHS[m.group("month")], int(m.group("day")), 
            int(m.group("hour")), int(m.group("minute")), int(m.group("second")))
       
        return LogEvent(
            file_path, offset, end_offset, dt, m.group("loglevel"), 
            m.group("source"), m.group("message"))        

class ScmServerEventParser(object):

    def __init__(self):
        date_format = "(?P<year>\d{4})-(?P<month>\d{2})-(?P<day>\d{2})"
        time_format = "(?P<hour>\d{2}):(?P<minute>\d{2}):(?P<second>\d{2}),(?P<ms>\d{3})"
        log_level_format = "\w+"
        source_format = "\S+"
        message_format = ".+\\n{0,1}"

        server_string = "(?P<date>" + date_format + \
            ")\s+(?P<time>" + time_format + \
            ")\s+(?P<loglevel>" + log_level_format + \
            ")\s+(?P<source>" + source_format + \
            ")\s+(?P<message>" + message_format + ")"

        self.__server = re.compile(server_string)

    def parse_event(self, line):
        m = self.__server.match(line.get_content())
        if m is not None:
            return self.__construct_event(
                line.get_file_path(), line.get_offset(),
                line.get_offset() + len(line.get_content()), m)
        else:
            return None

    def __construct_event(self, file_path, offset, end_offset, m):
        dt = datetime.datetime(
            int(m.group("year")), int(m.group("month")), int(m.group("day")), 
            int(m.group("hour")), int(m.group("minute")), int(m.group("second")), 
            int(m.group("ms")) * 1000)
        return LogEvent(
            file_path, offset, end_offset, dt, m.group("loglevel"), 
            m.group("source"), m.group("message"))

class GLogEventParser(object):

    def __init__(self):
        date_format = "(?P<month>\d{2})(?P<day>\d{2})"
        time_format = "(?P<hour>\d{2}):(?P<minute>\d{2}):(?P<second>\d{2})\.(?P<us>\d{6})"
        log_level_format = "[^\d]+"
        source_format = "[^\]]+"
        # thread IDs may be '58225' or 'Thread-188' depending on whether this is
        # a pure Glog message or a wrapper around a log4j message.
        thread_id = "[\w\-]+"
        message_format = ".+\\n{0,1}"

        glog_regex_string = "(?P<loglevel>" + log_level_format + \
            ")(?P<date>" + date_format + \
            ")\s+(?P<time>" + time_format + \
            ")\s+(?P<thread_id>" + thread_id + \
            ")\s+(?P<source>" + source_format + \
            ")]\s+(?P<message>" + message_format + ")"

        self.__glog_regex = re.compile(glog_regex_string)

    def parse_event(self, line):
        m = self.__glog_regex.match(line.get_content())
        if m is not None:
            return self.__construct_event(
                line.get_file_path(), line.get_offset(),
                line.get_offset() + len(line.get_content()), m)
        else:
            return None

    def __construct_event(self, file_path, offset, end_offset, m):
        # glog doesn't put year in its log messages. If the month is less
        # than or equal current month then assume current year, otherwise
        # assume last year
        year = datetime.datetime.now().year
        month = datetime.datetime.now().month
        log_month = int(m.group("month"))
        if log_month > month:
            year -= 1
        dt = datetime.datetime(
            year, log_month, int(m.group("day")),
            int(m.group("hour")), int(m.group("minute")), int(m.group("second")),
            int(m.group("us")))
        return LogEvent(
            file_path, offset, end_offset, dt, GLOG_LOG_LEVELS[m.group("loglevel")],
            m.group("source"), m.group("message"))


